/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is bbs2chreader.
 *
 * The Initial Developer of the Original Code is
 * flyson.
 * Portions created by the Initial Developer are Copyright (C) 2008
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *    flyson <flyson at users.sourceforge.jp>
 *    unyapi <webmaster at up-cat.net>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */


EXPORTED_SYMBOLS = ["ImepitaServerScript"];


const Ci = Components.interfaces;
const Cc = Components.classes;


function ImepitaServerScript(aRequestURL){
}

ImepitaServerScript.prototype = {
	start: function(aServerHandler){
		this.aServerHandler = aServerHandler;

		var strViewUrl = aServerHandler.requestURL.filePath.substring(9);
		if(!strViewUrl.match(/^http\:\/\/imepita\.jp\/\d{8}\/\d{6}/)){
			aServerHandler.sendErrorPage(404, "view url error : " + aServerHandler.requestURL.spec);
			return;
		}

		var oViewUri = Components.classes["@mozilla.org/network/standard-url;1"]
							.createInstance(Components.interfaces.nsIURI);
		oViewUri.spec = strViewUrl;

		try{
			this._channel = this.getHttpChannel(oViewUri);
			//this._channel.loadFlags = this._channel.LOAD_BYPASS_CACHE;
			//this._channel.setRequestHeader("Accept", "text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8", false);
			//this._channel.setRequestHeader("Cache-Control", "max-age=0", false);
			this._channel.requestMethod = "GET";
			this._channel.asyncOpen(this, this.aServerHandler);
		}catch(e){
			this.aServerHandler.sendErrorPage(404, "view http channel create error : '" + e + "' url : " + this.aServerHandler.requestURL.spec);
		}
	},

	cancel: function(){
		// FIXME: we should keep oImageStreamer as ImepitaServerScript's member to enable this
		//if(this.oImageStreamer) this.oImageStreamer.cancel();
		if(this._channel){
		 	try{
				this._channel.cancel(0);
			}catch(e){}
		}
	},

	// ********** ********* implements nsIStreamListener ********** **********
	onStartRequest: function(aRequest, aContext){
		this._bInputStream = Components.classes["@mozilla.org/binaryinputstream;1"]
								.createInstance(Components.interfaces.nsIBinaryInputStream);
		this._data = "";
	},

	onDataAvailable: function (aRequest, aContext, aInputStream, aOffset, aCount) {
		this._bInputStream.setInputStream(aInputStream);
		this._data += this._bInputStream.readBytes(aCount);
	},

	onStopRequest: function(aRequest, aContext, aStatus) {
		var oImageStreamer = new ImepitaImageStreamer();
		_bInputStream = null;
		if(aStatus == 0){
			var strImageUrl = this._data.match(/\/image\/\d{8}\/\d{6}[^\"]*/); //"
/*
			// Cookie: xxxxxxxxxxxxxxxxxxx
			// this is not required currently.
			// HttpChannel already has cookie created from system store,
			// so we need this if required cookie cheating.
			var strCookie = this._channel.getResponseHeader("Set-Cookie");
			if(strCookie && strImageUrl){
				oImageStreamer.start(this, this.aServerHandler, "http://imepita.jp/image/" + strImageUrl, strCookie);
				return;
			}
*/
			if(strImageUrl){
				oImageStreamer.start(this, this.aServerHandler, "http://imepita.jp" + strImageUrl);
			}else{
				this.aServerHandler.sendErrorPage(404, "image url not found : " + this.aServerHandler.requestURL.spec);
			}
		}else{
			this.aServerHandler.sendErrorPage(404, "view http channel result error : " + this.aServerHandler.requestURL.spec);
		}
		this._channel = null;
		this._data = "";
		this.aServerHandler = null;
	},
	// (static)
	getHttpChannel : function(strUrl){
		// obsoleted
		//var oBbs2chService = Components.classes["@mozilla.org/bbs2ch-service;1"]
		//				.getService(Components.interfaces.nsIBbs2chService);
		var oBbs2chService = Components.classes["@bbs2ch.sourceforge.jp/b2r-global-service;1"]
						.getService(Components.interfaces.b2rIGlobalService);
		var channel = oBbs2chService.getHttpChannel(strUrl);
		// override b2rIGlobalService' setting
		channel.setRequestHeader("User-Agent", "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9) Gecko/2008052906 Firefox/3.0", false);
		// other stuff, not needed?
		//channel.setRequestHeader("Accept-Language", "en-us,en;q=0.5", false);
		//channel.setRequestHeader("Accept-Encoding", "gzip,deflate", false);
		//channel.setRequestHeader("Accept-Charset", "ISO-8859-1,utf-8;q=0.7,*;q=0.7", false);
		//channel.setRequestHeader("Keep-Alive", "300", false);
		//channel.setRequestHeader("Connection", "keep-alive", false);

		return channel;
	}
};

function ImepitaImageStreamer(){
}
ImepitaImageStreamer.prototype = {
	start : function(oISScript, aServerHandler, strImageUrl, strCookie){
		this.aServerHandler = aServerHandler;
		var oImageUri = Components.classes["@mozilla.org/network/standard-url;1"]
							.createInstance(Components.interfaces.nsIURI);
		oImageUri.spec = strImageUrl;
		try{
			this._channel = oISScript.getHttpChannel(oImageUri);
			//this._channel.loadFlags = this._channel.LOAD_BYPASS_CACHE;
			//this._channel.setRequestHeader("Accept", "image/png,image/*;q=0.8,*/*;q=0.5", false);
			this._channel.setRequestHeader("Referer", this.aServerHandler.requestURL.filePath.substring(9), false);
			if(strCookie){
				this._channel.setRequestHeader("Cookie", strCookie, false);
			}
			this._channel.requestMethod = "GET";
			this._channel.asyncOpen(this, this.aServerHandler);
		}catch(e){
			this.aServerHandler.sendErrorPage(404, "image http channel create error : '" + e + "' url : " + this.aServerHandler.requestURL.spec);
		}
	},
	cancel: function(){
		if(this._channel){
		 	try{
				this._channel.cancel(0);
			}catch(e){}
		}
	},

	// ********** ********* implements nsIStreamListener ********** **********
	onStartRequest: function(aRequest, aContext){
		this._bInputStream = Components.classes["@mozilla.org/binaryinputstream;1"]
								.createInstance(Components.interfaces.nsIBinaryInputStream);
		this._data = "";
	},

	onDataAvailable: function (aRequest, aContext, aInputStream, aOffset, aCount) {
		this._bInputStream.setInputStream(aInputStream);
		this._data += this._bInputStream.readBytes(aCount);
	},

	onStopRequest: function(aRequest, aContext, aStatus) {
		_bInputStream = null;
		if(aStatus == 0){
			this._channel.visitResponseHeaders(this);
			this.aServerHandler.writeResponseHeader(this._channel.responseStatus);
			this.aServerHandler.write(this._data);
			this.aServerHandler.flush();
			this.aServerHandler.close();
		}else{
			this.aServerHandler.sendErrorPage(404, "image http channel result error : " + this.aServerHandler.requestURL.spec);
		}
		this._channel = null;
		this._data = "";
		this.aServerHandler = null;
	},

	// ********** ********* implements nsIHttpHeaderVisitor ********** **********
	visitHeader: function(aHeader, aValue){
		this.aServerHandler.setResponseHeader(aHeader, aValue);
	}
};
